@@ -95,84 +95,101 @@ impl Directive {
 struct AdmonitionInfoRaw<'a> {
     directive: &'a str,
     title: Option<String>,
+    additional_classnames: Option<Vec<&'a str>>,
 }
 
 #[derive(Debug, PartialEq)]
 struct AdmonitionInfo<'a> {
     directive: Directive,
-    title: Cow<'a, str>,
+    title: Option<String>,
+    additional_classnames: Option<Vec<&'a str>>,
 }
 
-impl<'a> Default for AdmonitionInfo<'a> {
-    fn default() -> Self {
+impl<'a> From<AdmonitionInfoRaw<'a>> for AdmonitionInfo<'a> {
+    fn from(other: AdmonitionInfoRaw<'a>) -> Self {
+        let AdmonitionInfoRaw {
+            directive: raw_directive,
+            title,
+            additional_classnames,
+        } = other;
+        let (directive, title) = match (Directive::from_str(raw_directive), title) {
+            (Ok(directive), None) => (directive, ucfirst(raw_directive)),
+            (Err(_), None) => (Directive::Note, "Note".to_owned()),
+            (Ok(directive), Some(title)) => (directive, title),
+            (Err(_), Some(title)) => (Directive::Note, title),
+        };
+        // If the user explicitly gave no title, then disable the title bar
+        let title = if title.is_empty() { None } else { Some(title) };
         Self {
-            directive: Directive::Note,
-            title: Cow::Borrowed("Note"),
-        }
-    }
-}
-
-impl<'a> TryFrom<AdmonitionInfoRaw<'a>> for AdmonitionInfo<'a> {
-    type Error = ();
-
-    fn try_from(other: AdmonitionInfoRaw<'a>) -> Result<Self, ()> {
-        let directive = Directive::from_str(other.directive)?;
-        Ok(Self {
             directive,
-            title: other
-                .title
-                .map(Cow::Owned)
-                .unwrap_or_else(|| Cow::Owned(ucfirst(other.directive))),
-        })
+            title,
+            additional_classnames,
+        }
     }
 }
 
 #[derive(Debug, PartialEq)]
 struct Admonition<'a> {
     directive: Directive,
-    title: Cow<'a, str>,
+    title: Option<String>,
     content: &'a str,
+    additional_classnames: Option<Vec<&'a str>>,
 }
 
 impl<'a> Admonition<'a> {
     pub fn new(info: AdmonitionInfo<'a>, content: &'a str) -> Self {
-        let AdmonitionInfo { directive, title } = info;
+        let AdmonitionInfo {
+            directive,
+            title,
+            additional_classnames,
+        } = info;
         Self {
             directive,
             title,
             content,
+            additional_classnames,
         }
     }
 
     fn html(&self) -> String {
-        let directive_classname = self.directive.classname();
+        let mut additional_class = Cow::Borrowed(self.directive.classname());
         let title = &self.title;
         let content = &self.content;
 
+        let title_html = title
+            .as_ref()
+            .map(|title| {
+                Cow::Owned(format!(
+                    r#"<div class="admonition-title">
+
+{title}
+
+</div>
+"#
+                ))
+            })
+            .unwrap_or(Cow::Borrowed(""));
+
+        if let Some(additional_classnames) = &self.additional_classnames {
+            let mut buffer = additional_class.into_owned();
+            for additional_classname in additional_classnames {
+                buffer.push(' ');
+                buffer.push_str(additional_classname);
+            }
+
+            additional_class = Cow::Owned(buffer);
+        }
+
         // Notes on the HTML template:
         // - the additional whitespace around the content are deliberate
         //   In line with the commonmark spec, this allows the inner content to be
         //   rendered as markdown paragraphs.
-        // - <p> nested in <div> is deliberate
-        //   - If plain text is given, it is contained in the <p> tag
-        //   - If markdown is given, it is rendered into a new <p> tag.
-        //     This leads to it escaping the template <p> tag, and to apply
-        //     styling we contain in in the outer <div>.
         format!(
-            r#"<div class="admonition {directive_classname}">
-<div class="admonition-title">
-<p>
-
-{title}
-
-</p>
-</div>
-<div>
-<p>
+            r#"<div class="admonition {additional_class}">
+{title_html}<div>
 
 {content}
 
-</p>
 </div>
 </div>"#,
         )
@@ -185,6 +202,7 @@ const ADMONISH_BLOCK_KEYWORD: &str = "admonish";
 /// - `None` if this is not an `admonish` block.
 /// - `Some(AdmonitionInfoRaw)` if this is an `admonish` block
 fn parse_info_string(info_string: &str) -> Option<AdmonitionInfoRaw> {
+    // Get the rest of the info string if this is an admonition
     let directive_title = if info_string == ADMONISH_BLOCK_KEYWORD {
         ""
     } else {
@@ -194,22 +212,39 @@ fn parse_info_string(info_string: &str) -> Option<AdmonitionInfoRaw> {
         }
     };
 
-    let info = if let Some((directive, title)) = directive_title.split_once(' ') {
-        // The title is expected to be a quoted JSON string
-        let title: String = serde_json::from_str(title)
-            .unwrap_or_else(|error| format!("Error parsing JSON string: {error}"));
-        AdmonitionInfoRaw {
+    // If we're just given the directive, handle that
+    let (directive, title) = directive_title
+        .split_once(' ')
+        .map(|(directive, title)| (directive, Some(title)))
+        .unwrap_or_else(|| (directive_title, None));
+
+    // The title is expected to be a quoted JSON string
+    // If parsing fails, output the error message as the title for the user to correct
+    let title = title.map(|title| {
+        serde_json::from_str::<String>(title)
+            .unwrap_or_else(|error| format!("Error parsing JSON string: {error}"))
+    });
+
+    // If the directive contains additional classes, parse them out
+    const CLASSNAME_SEPARATOR: char = '.';
+    let (directive, additional_classnames) = match directive.split_once(CLASSNAME_SEPARATOR) {
+        None => (directive, None),
+        Some((directive, additional_classnames)) => (
             directive,
-            title: Some(title),
-        }
-    } else {
-        AdmonitionInfoRaw {
-            directive: directive_title,
-            title: None,
-        }
+            Some(
+                additional_classnames
+                    .split(CLASSNAME_SEPARATOR)
+                    .filter(|additional_classname| !additional_classname.is_empty())
+                    .collect(),
+            ),
+        ),
     };
 
-    Some(info)
+    Some(AdmonitionInfoRaw {
+        directive,
+        title,
+        additional_classnames,
+    })
 }
 
 /// Make the first letter of `input` upppercase.
@@ -251,7 +286,7 @@ fn extract_admonish_body(content: &str) -> &str {
 /// If the code block is not an admonition, return `None`.
 fn parse_admonition<'a>(info_string: &'a str, content: &'a str) -> Option<Admonition<'a>> {
     let info = parse_info_string(info_string)?;
-    let info = AdmonitionInfo::try_from(info).unwrap_or_default();
+    let info = AdmonitionInfo::from(info);
     let body = extract_admonish_body(content);
     Some(Admonition::new(info, body))
 }
@@ -307,6 +342,7 @@ mod test {
             Some(AdmonitionInfoRaw {
                 directive: "",
                 title: None,
+                additional_classnames: None
             })
         );
         assert_eq!(
@@ -314,20 +350,31 @@ mod test {
             Some(AdmonitionInfoRaw {
                 directive: "",
                 title: None,
+                additional_classnames: None
             })
         );
         assert_eq!(
             parse_info_string("admonish unknown"),
             Some(AdmonitionInfoRaw {
                 directive: "unknown",
-                title: None
+                title: None,
+                additional_classnames: None
             })
         );
         assert_eq!(
             parse_info_string("admonish note"),
             Some(AdmonitionInfoRaw {
                 directive: "note",
-                title: None
+                title: None,
+                additional_classnames: None
+            })
+        );
+        assert_eq!(
+            parse_info_string("admonish note.additional-classname"),
+            Some(AdmonitionInfoRaw {
+                directive: "note",
+                title: None,
+                additional_classnames: Some(vec!["additional-classname"])
             })
         );
     }
@@ -345,18 +392,14 @@ Text
 
 <div class="admonition note">
 <div class="admonition-title">
-<p>
 
 Note
 
-</p>
 </div>
 <div>
-<p>
 
 A simple admonition.
 
-</p>
 </div>
 </div>
 Text
@@ -378,18 +421,14 @@ Text
 
 <div class="admonition warning">
 <div class="admonition-title">
-<p>
 
 Warning
 
-</p>
 </div>
 <div>
-<p>
 
 A simple admonition.
 
-</p>
 </div>
 </div>
 Text
@@ -411,18 +450,14 @@ Text
 
 <div class="admonition warning">
 <div class="admonition-title">
-<p>
 
 Read **this**!
 
-</p>
 </div>
 <div>
-<p>
 
 A simple admonition.
 
-</p>
 </div>
 </div>
 Text
@@ -508,18 +543,14 @@ hello
 
 <div class="admonition note">
 <div class="admonition-title">
-<p>
 
 And "<i>in</i>" the title
 
-</p>
 </div>
 <div>
-<p>
 
 With <b>html</b> styling.
 
-</p>
 </div>
 </div>
 hello
@@ -541,21 +572,92 @@ hello
 
 <div class="admonition warning">
 <div class="admonition-title">
-<p>
 
 Trademark™
 
-</p>
 </div>
 <div>
-<p>
 
 Should be respected
 
-</p>
 </div>
 </div>
 hello
+"#;
+
+        assert_eq!(expected, preprocess(content).unwrap());
+    }
+
+    #[test]
+    fn block_with_additional_classname() {
+        let content = r#"
+```admonish tip.my-style.other-style
+Will have bonus classnames
+```
+"#;
+
+        let expected = r#"
+
+<div class="admonition tip my-style other-style">
+<div class="admonition-title">
+
+Tip
+
+</div>
+<div>
+
+Will have bonus classnames
+
+</div>
+</div>
+"#;
+
+        assert_eq!(expected, preprocess(content).unwrap());
+    }
+
+    #[test]
+    fn block_with_additional_classname_and_title() {
+        let content = r#"
+```admonish tip.my-style.other-style "Developers don't want you to know this one weird tip!"
+Will have bonus classnames
+```
+"#;
+
+        let expected = r#"
+
+<div class="admonition tip my-style other-style">
+<div class="admonition-title">
+
+Developers don't want you to know this one weird tip!
+
+</div>
+<div>
+
+Will have bonus classnames
+
+</div>
+</div>
+"#;
+
+        assert_eq!(expected, preprocess(content).unwrap());
+    }
+
+    #[test]
+    fn block_with_empty_additional_classnames_title_content() {
+        let content = r#"
+```admonish .... ""
+```
+"#;
+
+        let expected = r#"
+
+<div class="admonition note">
+<div>
+
+
+
+</div>
+</div>
 "#;
 
         assert_eq!(expected, preprocess(content).unwrap());
